package plp.expressions1.expression;

import plp.expressions1.util.Tipo;

/**
 * Um objeto desta classe representa uma Expressao de Soma.
 */
public class ExpSoma extends ExpBinaria {

	/**
	 * Controi uma Expressao de Soma com as sub-expressoes especificadas.
	 * Assume-se que estas sub-expressoes resultam em <code>ValorInteiro</code> 
	 * quando avaliadas.
	 * @param esq Expressao da esquerda
	 * @param dir Expressao da direita
	 */
	public ExpSoma(Expressao esq, Expressao dir) {
		super(esq, dir, "+");
	}

	/**
	 * Retorna o valor da Expressao de Soma
	 */
	public Valor avaliar() {
		if (getEsq().getTipo() == Tipo.TIPO_REAL && getEsq().getTipo() == Tipo.TIPO_REAL) {
			return new ValorReal(
					((ValorReal) getEsq().avaliar()).valor() +
					((ValorReal) getDir().avaliar()).valor() );
		} else if (getEsq().getTipo() == Tipo.TIPO_INTEIRO && getEsq().getTipo() == Tipo.TIPO_INTEIRO) {
			return new ValorInteiro(
					((ValorInteiro) getEsq().avaliar()).valor() +
					((ValorInteiro) getDir().avaliar()).valor() );
		}
		
		return null;
	}
	
	/**
	 * Realiza a verificacao de tipos desta expressao.
	 *
	 * @return <code>true</code> se os tipos da expressao sao validos;
	 *          <code>false</code> caso contrario.
	 */
	protected boolean checaTipoElementoTerminal() {
		return (getEsq().getTipo().eInteiro() && getDir().getTipo().eInteiro()) 
			|| (getEsq().getTipo().eReal() && getDir().getTipo().eReal());
	}

	/**
	 * Retorna os tipos possiveis desta expressao.
	 *
	 * @return os tipos possiveis desta expressao.
	 */
	public Tipo getTipo() {
		if (getEsq().getTipo() == Tipo.TIPO_REAL) {
			return Tipo.TIPO_REAL;
		} else { 
			return Tipo.TIPO_INTEIRO;
		}
	}

}
