package plp.imperative1.command;

import plp.expressions2.expression.Expressao;
import plp.imperative1.memory.AmbienteExecucaoImperativa;
import plp.imperative1.memory.AmbienteCompilacaoImperativa;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.expressions2.memory.VariavelJaDeclaradaException;

public class Write implements IO {
    
    private Expressao expressao;
    
    public Write(Expressao expressao){
        this.expressao = expressao;
    }
    
    /**
     * Escreve na saida padro.
     *
     * @param ambiente o ambiente de execuo.
     *
     * @return o ambiente depois de modificado pela execuo
     * do comando <code>write</code>.
     *
     */
    public AmbienteExecucaoImperativa executar(AmbienteExecucaoImperativa ambiente) 
        throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException {
    	ambiente.write( expressao.avaliar(ambiente) );
        return ambiente;
    }

    /**
     * Realiza a verificacao de tipos da expresso a ser escrita na
     * pelo comando <code>write</code>
     *
     * @param ambiente o ambiente de compilao.
     * @return <code>true</code> se a expresso a ser escrita est bem tipada;
     *          <code>false</code> caso contrario.
     */
    public boolean checaTipo(AmbienteCompilacaoImperativa ambiente) 
        throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException {
        return expressao.checaTipo(ambiente);
    }


}
