package plp.orientadaObjetos1;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.comando.Comando;
import plp.orientadaObjetos1.declaracao.elemento.DecElemento;
import plp.orientadaObjetos1.excecao.declaracao.ElementoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.execucao.BaseDeRegraNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.execucao.CondicaoInvalidadaException;
import plp.orientadaObjetos1.excecao.execucao.EntradaInvalidaException;
import plp.orientadaObjetos1.excecao.execucao.EntradaNaoFornecidaException;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.memoria.colecao.ListaValor;
/**
 * Classe que representa um programa na linguagem OO.
 */
public class Programa {
    /**
     * Declaraao de classe
     */
    private DecElemento decClasse;
    /**
     * Comando executado aps a declaraao de classes
     */
    private Comando comando;

    /**
     * Construtor.
     * @param decClasse A declaraao de classe(s)
     * @param comando O comando executado aps a declaraao.
     */
    public Programa(DecElemento decClasse, Comando comando){
        this.decClasse = decClasse;
        this.comando = comando;
    }

     /**
     * Executa o programa.
     *
     * @param ambiente o ambiente de execuo.
     *
     * @return o ambiente depois de modificado pela execuo
     * do programa.
     *
     * @exception EntradaNaoFornecidaException se no for fornecida
     *  a tail de valores de entrada do programa.
     * @throws CondicaoInvalidadaException 
     * @throws BaseDeRegraNaoDeclaradaException 
     *
     */
    public ListaValor executar(AmbienteExecucaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException,
               ObjetoNaoDeclaradoException, ObjetoJaDeclaradoException,
               ProcedimentoJaDeclaradoException,ProcedimentoNaoDeclaradoException,
               ElementoJaDeclaradoException, ElementoNaoDeclaradoException,
               EntradaNaoFornecidaException, EntradaInvalidaException, CondicaoInvalidadaException, BaseDeRegraNaoDeclaradaException {
        if(ambiente == null)
            throw new EntradaNaoFornecidaException();
        ambiente.incrementa();
        ambiente = comando.executar(decClasse.elabora(ambiente));
        ambiente.restaura();
        return ambiente.getSaida();
    }

    /**
     * Realiza a verificacao de tipos do programa
     *
     * @param ambiente o ambiente de compilao.
     * @return <code>true</code> se o programa est bem tipado;
     *          <code>false</code> caso contrario.
     *
     * @exception EntradaNaoFornecidaException se no for fornecida
     *  a tail de valores de entrada do programa.
     *
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException,
               ProcedimentoNaoDeclaradoException, ProcedimentoJaDeclaradoException,
               ElementoJaDeclaradoException, ElementoNaoDeclaradoException,
               EntradaNaoFornecidaException{
        boolean resposta;
        if(ambiente == null) {
            throw new EntradaNaoFornecidaException();
        }
        ambiente.incrementa();
        resposta = decClasse.checaTipo(ambiente) && comando.checaTipo(ambiente);
        ambiente.restaura();
        return resposta;
    }
}