package plp.orientadaObjetos1.agente;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.imperative1.util.Lista;
import plp.orientadaObjetos1.comando.Comando;
import plp.orientadaObjetos1.declaracao.procedimento.ListaDeclaracaoParametro;
import plp.orientadaObjetos1.declaracao.variavel.DecVariavel;
import plp.orientadaObjetos1.excecao.declaracao.ElementoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.execucao.BaseDeRegraNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.execucao.CondicaoInvalidadaException;
import plp.orientadaObjetos1.excecao.execucao.EntradaInvalidaException;
import plp.orientadaObjetos1.expressao.Expressao;
import plp.orientadaObjetos1.expressao.ListaExpressao;
import plp.orientadaObjetos1.expressao.leftExpression.Id;
import plp.orientadaObjetos1.expressao.valor.Valor;
import plp.orientadaObjetos1.expressao.valor.ValorBooleano;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;


public class CorpoRegra {
	
	private ListaDeclaracaoParametro parametros; // Os parmetros necessrios para a execuso da regra
	private DecVariavel variavel; // Variveis locais da regra
	private ListaExpressao condicoes; // As condies necessrias a execusso da regra
	private Comando acoes; // As aes da regra
	
	// TODO remover
	private Id nomeRegra;

	/**
	 * Construtor com os atributos da classe
	 * 
	 * @param parametros     Os parmetros necessrios para a execuso da regra 
	 * @param variavel       Variveis locais da regra
	 * @param listaExpressao As condies necessrias a execusso da regra 
	 * @param comando        As aes da regra
	 */
	public CorpoRegra(ListaDeclaracaoParametro parametros, DecVariavel variavel, ListaExpressao listaExpressao, Comando comando) {
		this.parametros = parametros;
		this.variavel = variavel;
		this.condicoes = listaExpressao;
		this.acoes = comando;
	}

	/**
	 * Verifica a validade do tipo dos componenes da regra
	 * 
	 * @param ambiente O ambiente de execuo
	 * 
	 * @return true se os tipos forem coerentes, false caso contrrio
	 * 
	 */
	public boolean checaTipo(AmbienteCompilacaoOO1 ambiente) throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException, ProcedimentoJaDeclaradoException, ProcedimentoNaoDeclaradoException, ElementoJaDeclaradoException, ElementoNaoDeclaradoException {
		
		boolean retorno = this.parametros.checaTipo(ambiente);
		
		if (!retorno) {
			return retorno;
		}
		
		ambiente.incrementa();
		ambiente.mapParametrosProcedimento(nomeRegra, parametros);
		parametros.declaraParametro(ambiente);
		
		retorno = this.variavel.checaTipo(ambiente) && this.acoes.checaTipo(ambiente);
		
		if (!retorno) {
			return retorno;
		}
		
		Lista<Expressao> temp = this.condicoes;  
		
		while(temp != null && temp.getHead() != null && retorno) {
			retorno = retorno && temp.getHead().checaTipo(ambiente);
			temp = temp.getTail();			
		}
		
		ambiente.restaura();
        
        return retorno;
		
	}
	
	/**
	 * Verifica se cada uma das condies da regra so verdadeiras.
	 * 
	 * @param ambiente O ambiente de execuo
	 * 
	 * @return true se todas as regras sultarem em true, false caso contrrio
	 * 
	 */	
	public boolean verificarCondicoes(AmbienteExecucaoOO1 ambiente) throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException, ObjetoNaoDeclaradoException, ElementoNaoDeclaradoException, CondicaoInvalidadaException {
		boolean retorno = true;
		
		Lista<Expressao> temp = this.condicoes;
		
		while(temp != null && temp.getHead() != null&& retorno) {
			Valor resultado = temp.getHead().avaliar(ambiente);
			
			if (resultado instanceof ValorBooleano) {
				retorno = retorno && (((ValorBooleano) resultado).valor());
			} else {
				throw new CondicaoInvalidadaException("Condio de regra invlida");
			}
			
			temp = temp.getTail();			
		}
		
		return retorno;
	}
	
	/**
	 * 
	 * Executa as aes da regra.
	 * 
	 * @param ambiente O ambiente de execuo
	 * 
	 * @return O ambiente resultante de execuo  
	 * @throws BaseDeRegraNaoDeclaradaException 
	 */
	public AmbienteExecucaoOO1 excecutar(AmbienteExecucaoOO1 ambiente) throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException, ObjetoJaDeclaradoException, ObjetoNaoDeclaradoException, ProcedimentoNaoDeclaradoException, ProcedimentoJaDeclaradoException, ElementoJaDeclaradoException, ElementoNaoDeclaradoException, EntradaInvalidaException, CondicaoInvalidadaException, BaseDeRegraNaoDeclaradaException {
		return this.acoes.executar(ambiente);
	}

	public ListaDeclaracaoParametro getParametros() {
		return parametros;
	}

	public Id getNomeRegra() {
		return nomeRegra;
	}

	public void setNomeRegra(Id nomeRegra) {
		this.nomeRegra = nomeRegra;
	}

	public DecVariavel getVariavel() {
		return variavel;
	}
	
}