package plp.orientadaObjetos1.comando;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.expressao.Expressao;
import plp.orientadaObjetos1.expressao.leftExpression.AcessoAtributo;
import plp.orientadaObjetos1.expressao.leftExpression.Id;
import plp.orientadaObjetos1.expressao.leftExpression.LeftExpression;
import plp.orientadaObjetos1.expressao.valor.ValorRef;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.memoria.Objeto;
import plp.orientadaObjetos1.util.TipoElemento;
/**
 * Classe que representa um comando de atribuio.
 */
public class Atribuicao implements Comando {
	/**
	 * Lado esquerdo do comando de atribuio.
	 */
    private LeftExpression av;
	/**
	 * Expresso cujo valor ser atribudo ao lado esquerdo.
	 */
    private Expressao expressao;
	/**
	 * Construtor.
	 * @param av Lado esquerdo
	 * @param expressao Expresso cujo valor ser atribudo ao lado esquerdo.
	 */
    public Atribuicao(LeftExpression av, Expressao expressao){
        this.av = av;
        this.expressao = expressao;
    }

    /**
     * Executa  a atribuio.
     *
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     *  e valores.
     * @return o ambiente modificado pela execuo da atribuio.
     * @throws ElementoNaoDeclaradoException 
     *
     */
    public AmbienteExecucaoOO1 executar(AmbienteExecucaoOO1 ambiente)
        throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
               ObjetoNaoDeclaradoException, ElementoNaoDeclaradoException {

        Id idVariavel =  av.getId();
        if ( av instanceof AcessoAtributo){
            // se for acesso a atributo, tem de alterar o ambiente do objeto!
            Expressao expAV = ((AcessoAtributo)av).getExpressaoObjeto();
            ValorRef referencia = (ValorRef)expAV.avaliar(ambiente);
            Objeto obj = ambiente.getObjeto(referencia);
            AmbienteExecucaoOO1 aux = obj.getEstado();                   // recuperando o ambiente do objeto
            aux.changeValor(idVariavel, expressao.avaliar(ambiente)); // alterando o ambiente do objeto
        }
        else
            ambiente.changeValor(idVariavel, expressao.avaliar(ambiente));
        return ambiente;
    }

    /**
     * Um comando de atribuio est bem tipado, se o tipo do identificador 
     * o mesmo da expresso. O tipo de um identificador  determinado pelo
     * tipo da expresso que o inicializou (na declarao).
     *
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     *  e valores.
     * @return <code>true</code> se os tipos da atribuio so vlidos;
     *          <code>false</code> caso contrario.
     *
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, ElementoNaoDeclaradoException {
        return expressao.checaTipo(ambiente) &&
              (av.getTipo(ambiente).equals(expressao.getTipo(ambiente))
               || expressao.getTipo(ambiente).equals(TipoElemento.TIPO_NULL));
    }
}