package plp.orientadaObjetos1.comando;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.execucao.BaseDeRegraNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.execucao.CondicaoInvalidadaException;
import plp.orientadaObjetos1.excecao.execucao.EntradaInvalidaException;
import plp.orientadaObjetos1.expressao.Expressao;
import plp.orientadaObjetos1.expressao.ListaExpressao;
import plp.orientadaObjetos1.expressao.leftExpression.Id;
import plp.orientadaObjetos1.expressao.valor.ValorRef;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.memoria.ContextoExecucaoOO1;
import plp.orientadaObjetos1.memoria.DefElemento;
import plp.orientadaObjetos1.memoria.Objeto;
import plp.orientadaObjetos1.memoria.colecao.ListaValor;
import plp.orientadaObjetos1.util.Tipo;

/**
 * Classe que representa a chamada de um mtodo.
 */
public class ChamadaMetodo implements Comando {
	/**
	 * A expresso que chama o mtodo.
	 */
    private Expressao expressao;
    /**
     * O identificador que representa o nome do mtodo.
     */
    private Id nomeMetodo;
    /**
     * Parmetros passados para o mtodo.
     */
    private ListaExpressao parametrosReais;

    /**
     * Construtor.
     * @param expressao A expressao chamadora do mtodo.
     * @param nomeMetodo O nome do mtodo.
     * @param parametrosReais Os parmetros passados para a execuo do mtodo.
     * @param
     */
    public ChamadaMetodo(Expressao expressao, Id nomeMetodo, ListaExpressao parametrosReais){
        this.expressao = expressao;
        this.nomeMetodo = nomeMetodo;
        this.parametrosReais = parametrosReais;
    }
	/**
	 * Executa uma chamada de mtodo.
	 * @param ambiente O ambiente de execuo, que guarda o mapeamento
	 * de identificadores a valores.
	 * @return o Ambiente de Execuo atualizado.
	 * @throws CondicaoInvalidadaException 
	 * @throws BaseDeRegraNaoDeclaradaException 
	 */
    public AmbienteExecucaoOO1 executar(AmbienteExecucaoOO1 ambiente)
        throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
        ProcedimentoNaoDeclaradoException, ProcedimentoJaDeclaradoException,
        ObjetoJaDeclaradoException, ObjetoNaoDeclaradoException,
        ElementoNaoDeclaradoException, ElementoJaDeclaradoException, EntradaInvalidaException, CondicaoInvalidadaException, BaseDeRegraNaoDeclaradaException{

        ValorRef vr = (ValorRef) expressao.avaliar(ambiente);  // recupera o id do objeto
        Objeto objeto =  ambiente.getObjeto(vr);               // recupera o objeto
        Id idClasse = objeto.getClasse(); 
        // recupera o tipo do objeto
        DefElemento defElemento = ambiente.getDefElemento(idClasse); // recupera a definio da classe
        Procedimento metodo = defElemento.getMetodo(nomeMetodo); // recupera o procedimento
        // cria um novo ambiente para a execucao, pois
        // no deve levar em conta as variveis definidas na main
        AmbienteExecucaoOO1 aux = new ContextoExecucaoOO1(ambiente);
                                                               //  change pois no construtor do ambiente
        //aux.changeValor(new Id("$this$"),vr);                    // invocado na linha anterior ja  feito
                                                               //  um mapeamento
        aux.map(new Id("this"), vr);
//        aux.changeValor(idClasse,vr);              
        //aux.incrementa();
        //aux.mapObjeto(vr, objeto);
        
        //objeto.g
        
        
        ListaValor valoresDosParametros = parametrosReais.avaliar(ambiente);
        
        ChamadaProcedimento cp = new ChamadaProcedimento(metodo, parametrosReais, valoresDosParametros);         
        if (this.isThread()) {
        	(new ThreadMethod(aux, cp)).start();
        }else{
        	cp.executar(aux);
        	//aux.restaura();
        }
                
        return ambiente;
    }
    
    private boolean isThread(){
    	return this.nomeMetodo.getIdName().equals("run") && parametrosReais.length()==0;
    }
    

    /**
    * Realiza a verificao de tipos desta chamada de mtodo, onde
    * o tipo do mtodo deve estar na definio da classe obtida
    * a partir de expresso.
    *
    * @param ambiente o ambiente que contem o mapeamento entre identificadores
    *  e tipos.
    * @return <code>true</code> se a chamada de mtodo est bem tipada;
    *          <code>false</code> caso contrario.
    */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
    throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException,
         ElementoNaoDeclaradoException {
        boolean resposta;
        //Antes de incrementar o ambiente, verifico se o mtodo
        // vlido para a definicao de classe obtida a partir de expressao.
        //Se no for vlido, a exceo ProcedimentoNaoDeclaradoException ser
        //lanada e checaTipo retornar false.
        Tipo tipoClasse = expressao.getTipo(ambiente);
        DefElemento defElemento = ambiente.getDefElemento(tipoClasse.getTipo());
        try{
            Procedimento metodo = defElemento.getMetodo(nomeMetodo);
            ambiente.incrementa();
            ambiente.map(new Id("this"),tipoClasse);
            resposta = new ChamadaProcedimento(metodo, parametrosReais).checaTipo(ambiente);
            ambiente.restaura();
        }
        catch(ProcedimentoNaoDeclaradoException e){
            resposta = false;
        }
        return resposta;
    }
}

class ThreadMethod extends Thread {
	private AmbienteExecucaoOO1 ambienteDeExecucao;
	private ChamadaProcedimento chamadaDeProcedimento;

	public ThreadMethod(AmbienteExecucaoOO1 ambienteDeExecucao, ChamadaProcedimento chamadaDeProcedimento) {
		super();
		this.ambienteDeExecucao = ambienteDeExecucao;
		this.chamadaDeProcedimento = chamadaDeProcedimento;
	}

	@Override
	public void run() {
		try {
			chamadaDeProcedimento.executar(ambienteDeExecucao);
			//this.ambienteDeExecucao.restaura();
		} catch (Throwable t) {
			throw new RuntimeException(t);
		}
	}	
	
}
