package plp.orientadaObjetos1.comando;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.agente.CorpoRegra;
import plp.orientadaObjetos1.declaracao.procedimento.ListaDeclaracaoParametro;
import plp.orientadaObjetos1.excecao.declaracao.ElementoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.execucao.BaseDeRegraNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.execucao.CondicaoInvalidadaException;
import plp.orientadaObjetos1.excecao.execucao.EntradaInvalidaException;
import plp.orientadaObjetos1.expressao.leftExpression.Id;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.memoria.colecao.ListaValor;

public class ChamadaRegra implements Comando {
	
	private Id nomeAgente;
	
	private Id nomeRegra;
	
	private CorpoRegra corpo;
	
    private ListaValor valoresParametros;

	public ChamadaRegra(Id nomeAgente, Id nomeRegra, CorpoRegra corpo, ListaValor valoresParametros) {
		super();
		this.nomeAgente = nomeAgente;
		this.nomeRegra = nomeRegra;
		this.corpo = corpo;
		this.valoresParametros = valoresParametros;
	}

	public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
			throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException,
			ProcedimentoNaoDeclaradoException,
			ProcedimentoJaDeclaradoException, ElementoJaDeclaradoException,
			ElementoNaoDeclaradoException {
		return corpo.checaTipo(ambiente);
	}

	public AmbienteExecucaoOO1 executar(AmbienteExecucaoOO1 ambiente)
			throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
			ObjetoJaDeclaradoException, ObjetoNaoDeclaradoException,
			ProcedimentoNaoDeclaradoException,
			ProcedimentoJaDeclaradoException, ElementoJaDeclaradoException,
			ElementoNaoDeclaradoException, EntradaInvalidaException, CondicaoInvalidadaException, BaseDeRegraNaoDeclaradaException {

		ambiente.incrementa();
		
		this.bindParameters(ambiente, corpo.getParametros());
		this.corpo.getVariavel().elabora(ambiente);
		
		if (this.corpo.verificarCondicoes(ambiente)) {
			this.corpo.excecutar(ambiente);
		}
		
		ambiente.restaura();
		
		return ambiente;
	}
	
	/**
     * insere no contexto o resultado da associacao entre cada parametro formal
     * e seu correspondente parametro atual
     * @throws ElementoNaoDeclaradoException 
     */
    private AmbienteExecucaoOO1 bindParameters (AmbienteExecucaoOO1 ambiente, ListaDeclaracaoParametro parametros)
        throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
               ObjetoNaoDeclaradoException, ElementoNaoDeclaradoException{
        
    	ListaValor listaValor = this.valoresParametros;
        
        while (listaValor.length() > 0){
            ambiente.map(parametros.getHead().getId(), listaValor.getHead());
            parametros = ((ListaDeclaracaoParametro) parametros.getTail());
            listaValor = (ListaValor)listaValor.getTail();
        }

        return ambiente;
    }

}
