package plp.orientadaObjetos1.comando;

import java.util.List;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.imperative1.util.Lista;
import plp.orientadaObjetos1.agente.BaseDeRegras;
import plp.orientadaObjetos1.agente.CorpoRegra;
import plp.orientadaObjetos1.agente.DecRegras;
import plp.orientadaObjetos1.agente.DecRegrasAtomica;
import plp.orientadaObjetos1.agente.DecRegrasComposta;
import plp.orientadaObjetos1.declaracao.procedimento.DecParametro;
import plp.orientadaObjetos1.excecao.declaracao.ElementoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.execucao.BaseDeRegraNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.execucao.CondicaoInvalidadaException;
import plp.orientadaObjetos1.excecao.execucao.EntradaInvalidaException;
import plp.orientadaObjetos1.expressao.leftExpression.Id;
import plp.orientadaObjetos1.expressao.valor.ValorRef;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.memoria.DefAgente;
import plp.orientadaObjetos1.memoria.DefElemento;
import plp.orientadaObjetos1.memoria.Objeto;
import plp.orientadaObjetos1.memoria.colecao.ListaValor;

public class ComandoIniciar extends ComandoAgente implements Runnable {

	private AmbienteExecucaoOO1 amb;
	private List<ValorRef> fatos;
	private BaseDeRegras base;
	
	public ComandoIniciar(Id nomeBase, Id objeto) {
		super(nomeBase, objeto);
	}

	private void dispararRegras(AmbienteExecucaoOO1 ambiente,
			List<ValorRef> fatos, BaseDeRegras regras)
			throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
			ObjetoJaDeclaradoException, ObjetoNaoDeclaradoException,
			ProcedimentoNaoDeclaradoException,
			ProcedimentoJaDeclaradoException, ElementoJaDeclaradoException,
			ElementoNaoDeclaradoException, EntradaInvalidaException,
			CondicaoInvalidadaException, BaseDeRegraNaoDeclaradaException {

		dispararRegra(ambiente, fatos, regras.getRegras());
	}

	private void dispararRegra(AmbienteExecucaoOO1 ambiente,
			List<ValorRef> fatos, DecRegras decRegras)
			throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
			ObjetoJaDeclaradoException, ObjetoNaoDeclaradoException,
			ProcedimentoNaoDeclaradoException,
			ProcedimentoJaDeclaradoException, ElementoJaDeclaradoException,
			ElementoNaoDeclaradoException, EntradaInvalidaException,
			CondicaoInvalidadaException, BaseDeRegraNaoDeclaradaException {

		if (decRegras instanceof DecRegrasAtomica) {
			DecRegrasAtomica regra = (DecRegrasAtomica) decRegras;
			ListaValor valoresParametros = new ListaValor();
			procurarParamentros(ambiente, fatos, regra, valoresParametros,
					regra.getCorpo().getParametros());
		} else {
			DecRegrasComposta regras = (DecRegrasComposta) decRegras;

			dispararRegra(ambiente, fatos, regras.getRegra1());
			dispararRegra(ambiente, fatos, regras.getRegra2());
		}
	}

	private boolean procurarParamentros(AmbienteExecucaoOO1 ambiente,
			List<ValorRef> fatos, DecRegrasAtomica regra,
			ListaValor valoresParametros, Lista<DecParametro> parametros)
			throws ElementoNaoDeclaradoException, ObjetoNaoDeclaradoException, VariavelJaDeclaradaException, VariavelNaoDeclaradaException, ObjetoJaDeclaradoException, ProcedimentoNaoDeclaradoException, ProcedimentoJaDeclaradoException, ElementoJaDeclaradoException, EntradaInvalidaException, CondicaoInvalidadaException, BaseDeRegraNaoDeclaradaException {

		DecParametro decParametro = parametros.getHead();
		boolean retorno = false;
		
		for (ValorRef fato : fatos) {
		
			CorpoRegra corpo = regra.getCorpo();
			Objeto objeto = ambiente.getObjeto(fato);

			if (objeto.getClasse().equals(decParametro.getTipo().getTipo())) {

				// Escreve em valoresParametros
				valoresParametros.write(fato);

				if (valoresParametros.length() == corpo.getParametros().length()) {
					ChamadaRegra chamada = new ChamadaRegra(null, null, corpo, valoresParametros);
					chamada.executar(ambiente);
					retorno = true;
				} else {
					retorno = procurarParamentros(ambiente, fatos, regra, valoresParametros,
						parametros.getTail());
					
					if (!retorno) {
						return retorno;
					}
				}
				
				// apaga de valoresParametros
				valoresParametros.eraseLast();
			}
		}
		
		return retorno;

	}

	@Override
	public AmbienteExecucaoOO1 executar(AmbienteExecucaoOO1 ambiente)
			throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
			ObjetoJaDeclaradoException, ObjetoNaoDeclaradoException,
			ProcedimentoNaoDeclaradoException,
			ProcedimentoJaDeclaradoException, ElementoJaDeclaradoException,
			ElementoNaoDeclaradoException, EntradaInvalidaException,
			BaseDeRegraNaoDeclaradaException, CondicaoInvalidadaException {

		this.amb = ambiente;
		
		Objeto objeto = getObjetoPorId(amb, getNomeAgente());
		DefElemento elemento = amb.getDefElemento(objeto.getClasse());

		if (elemento instanceof DefAgente) {
			DefAgente agente = (DefAgente) elemento;

			this.base   = agente.getBaseDeRegras(getObjeto());
			this.fatos = agente.getFatos();
			Thread t = new Thread(this);
			t.start();
			
			
		}

				return ambiente;
	}

	public void run() {
		
		try {
			if(amb != null && fatos != null && base != null){
				dispararRegras(amb, fatos, base);	
			}
			
		} catch (Throwable t) {
			throw new RuntimeException(t);
		}
	}

}
