package plp.orientadaObjetos1.comando;

/*
 * A execucao de um comando ocorre em um determinado ambiente. O
 * resultado de tal execucao  a modificao deste ambiente, i.e., comandos
 * tem efeito colateral.
 */
import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.agente.DecAgenteAtomica;
import plp.orientadaObjetos1.agente.DecBaseDeRegras;
import plp.orientadaObjetos1.declaracao.variavel.DecVariavel;
import plp.orientadaObjetos1.excecao.declaracao.ElementoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.expressao.leftExpression.Id;
import plp.orientadaObjetos1.expressao.leftExpression.LeftExpression;
import plp.orientadaObjetos1.expressao.valor.ValorRef;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.memoria.ContextoExecucaoOO1;
import plp.orientadaObjetos1.memoria.DefAgente;
import plp.orientadaObjetos1.memoria.DefClasse;
import plp.orientadaObjetos1.memoria.DefElemento;
import plp.orientadaObjetos1.memoria.Objeto;
import plp.orientadaObjetos1.util.TipoElemento;

/**
 * Comando de criao de objeto e atribuio deste a uma expresso esquerda.
 */
public class New implements Comando {
	/**
	 * Lado esquerdo da atribuio.
	 */
	private LeftExpression av;

	/**
	 * Identificador da classe, com o seu nome.
	 */
	private Id classe;

	/**
	 * Construtor.
	 * 
	 * @param av
	 *            Lado esquerdo da atribuio.
	 * @param classe
	 *            Identificador com o nome da classe.
	 */
	public New(LeftExpression av, Id classe) {
		this.av = av;
		this.classe = classe;
	}

	/**
	 * Execuo da atribuio de um novo objeto criado a uma left expression.
	 * 
	 * @param ambiente
	 *            O ambiente contendo o mapeamento entre identificadores e
	 *            valores.
	 * @return o ambiente de execuo atualizado.
	 */
	public AmbienteExecucaoOO1 executar(AmbienteExecucaoOO1 ambiente)
			throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
			ElementoJaDeclaradoException, ElementoNaoDeclaradoException,
			ObjetoJaDeclaradoException, ObjetoNaoDeclaradoException {

		AmbienteExecucaoOO1 aux = null;
		DefElemento defClasse = (DefElemento) ambiente.getDefElemento(classe);
		if (defClasse instanceof DefClasse) {
			DecVariavel decVariavel = ((DefClasse) defClasse).getDecVariavel();
			aux = decVariavel.elabora(new ContextoExecucaoOO1(ambiente));
		} else {
			// TODO Adicionar suporte a base de regras
			aux = new ContextoExecucaoOO1(ambiente);
		}

		Objeto objeto = new Objeto(classe, aux.getContextoIdValor());
		ValorRef vr = ambiente.getProxRef();
		ambiente.mapObjeto(vr, objeto);
		ambiente = new Atribuicao(av, vr).executar(ambiente);

		return ambiente;
	}

	/**
	 * Verifica se a atribuio  possvel comparando os tipos do objeto e da
	 * left expression.
	 * 
	 * @param ambiente
	 *            O ambiente de compilao, com o mapeamento entre
	 *            identificadores e tipos.
	 */
	public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
			throws VariavelNaoDeclaradaException, ElementoJaDeclaradoException,
			ElementoNaoDeclaradoException {
		TipoElemento tpClasse = new TipoElemento(classe, TipoElemento.CLASSE);
		return av.checaTipo(ambiente) && tpClasse.eValido(ambiente)
				&& tpClasse.equals(av.getTipo(ambiente));
	}
}