package plp.orientadaObjetos1.comando;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.execucao.BaseDeRegraNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.execucao.CondicaoInvalidadaException;
import plp.orientadaObjetos1.excecao.execucao.EntradaInvalidaException;
import plp.orientadaObjetos1.expressao.Expressao;
import plp.orientadaObjetos1.expressao.valor.ValorBooleano;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.util.TipoPrimitivo;

/**
 * Classe que representa um comando While.
 */
public class While implements Comando{
	/**
	 * Expresso booleana a ser avaliada.
	 */
    private Expressao expressao;
	/**
	 * Comando que ser executado caso a expresso seja avaliada como verdadeira.
	 */
    private Comando comando;
	/**
	 * Construtor.
	 * @param expressao A expresso booleana a ser avaliada.
	 * @param o comando a ser executado caso a expresso seja verdadeira.
	 */
    public While(Expressao expressao, Comando comando){
        this.expressao  = expressao;
        this.comando = comando;
    }

    /**
     * Implementa o comando <code>while</code>.
     * @param ambiente o ambiente de execuo.
     * @return o ambiente depois de modificado pela execuo
     * do comando <code>while</code>.
     * @throws CondicaoInvalidadaException 
     * @throws BaseDeRegraNaoDeclaradaException 
     */
    public AmbienteExecucaoOO1 executar(AmbienteExecucaoOO1 ambiente)
        throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
               ProcedimentoNaoDeclaradoException, ProcedimentoJaDeclaradoException,
                ObjetoJaDeclaradoException, ObjetoNaoDeclaradoException,
                ElementoJaDeclaradoException, ElementoNaoDeclaradoException, EntradaInvalidaException, CondicaoInvalidadaException, BaseDeRegraNaoDeclaradaException{
        while ( ((ValorBooleano)expressao.avaliar(ambiente)).valor() ) {
            ambiente = comando.executar(ambiente);
        }
        return ambiente;
    }

    /**
     * Realiza a verificacao de tipos da expresso e dos
     * comandos do comando <code>while</code>
     * @param ambiente o ambiente de compilao.
     * @return <code>true</code> se os comando so bem tipados;
     *          <code>false</code> caso contrario.
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException,
        ProcedimentoNaoDeclaradoException, ProcedimentoJaDeclaradoException,
        ElementoNaoDeclaradoException, ElementoJaDeclaradoException {
        return expressao.checaTipo(ambiente) &&
               ((TipoPrimitivo)expressao.getTipo(ambiente)).eBooleano() &&
               comando.checaTipo(ambiente);
    }

}
