package plp.orientadaObjetos1.comando;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.expressao.Expressao;
import plp.orientadaObjetos1.expressao.valor.Valor;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;

/**
 * Comando de escrita.
 */
public class Write implements IO {
	/**
	 * Expresso a ser escrita.
	 */
    private Expressao expressao;
	/**
	 * Construtor.
	 * @param expresso Expresso a ser escrita.
	 */
    public Write(Expressao expressao){
        this.expressao = expressao;
    }

    /**
     * Escreve na saida padro.
     * @param ambiente o ambiente de execuo.
     * @return o ambiente depois de modificado pela execuo
     * do comando <code>write</code>.
     * @throws ElementoNaoDeclaradoException 
     */
    public AmbienteExecucaoOO1 executar(AmbienteExecucaoOO1 ambiente)
        throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
        ObjetoNaoDeclaradoException, ElementoNaoDeclaradoException {
        Valor valor = expressao.avaliar(ambiente);
        System.out.println(valor);
        return ambiente.write( valor);
    }

    /**
     * Realiza a verificacao de tipos da expresso a ser escrita na
     * pelo comando <code>write</code>
     * @param ambiente o ambiente de compilao.
     * @return <code>true</code> se a expresso a ser escrita est bem tipada;
     *          <code>false</code> caso contrario.
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
        throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
        ElementoNaoDeclaradoException {
        return expressao.checaTipo(ambiente);
    }
}
