package plp.orientadaObjetos1.declaracao.elemento;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.declaracao.procedimento.DecProcedimento;
import plp.orientadaObjetos1.declaracao.variavel.DecVariavel;
import plp.orientadaObjetos1.excecao.declaracao.ElementoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoNaoDeclaradoException;
import plp.orientadaObjetos1.expressao.leftExpression.Id;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.memoria.DefClasse;
import plp.orientadaObjetos1.util.TipoElemento;
/**
 * Classe que representa a declarao de uma nica classe.
 */
public class DecClasseSimples extends DecElementoSimples {
	
	/**
	 * Atributos da classe.
	 */
    private DecVariavel atributos;
	
	/**
	 * Construtor.
	 * @param nomeClasse Nome da classe
	 * @param atributos Atributos da classe
	 * @param metodos Mtodos da classe.
	 */
    public  DecClasseSimples(Id nomeClasse, DecVariavel atributos, DecProcedimento metodos){
        super(nomeClasse, metodos);
        this.atributos = atributos;
    }

    /**
     * Cria um mapeamento do identificador para a declarao desta classe.
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     *  e valores.
     * @return o ambiente modificado pela declarao da classe.
     */
    public AmbienteExecucaoOO1 elabora(AmbienteExecucaoOO1 ambiente)
        throws ElementoJaDeclaradoException, ElementoNaoDeclaradoException {

        ambiente.mapDefElemento(getNomeElemento(), new DefClasse(atributos,getMetodos()));
        return ambiente;
    }
    /**
     * Verifica se a declarao est bem tipada, ou seja, se a checagem dos tipos
     * dos mtodos e atributos est ok.
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     *  e seus tipos.
     * @return <code>true</code> se os tipos da declarao so vlidos;
     *          <code>false</code> caso contrario.
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
        throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
               ElementoJaDeclaradoException, ElementoNaoDeclaradoException,
               ProcedimentoNaoDeclaradoException, ProcedimentoJaDeclaradoException {

        ambiente.mapDefElemento(getNomeElemento(), new DefClasse(atributos,getMetodos()));
        boolean resposta = false;
        ambiente.incrementa();
        if (atributos.checaTipo(ambiente)){
            ambiente.map(new Id("this"), new TipoElemento(getNomeElemento(), TipoElemento.CLASSE));
            resposta =  getMetodos().checaTipo(ambiente);
        }
        ambiente.restaura();
        return resposta;
    }
}