package plp.orientadaObjetos1.declaracao.elemento;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoNaoDeclaradoException;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;

/**
 * Classe representando a declarao de mais de uma classe.
 */
public class DecElementoComposta implements DecElemento {
	/**
	 * Primeira declarao de classe.
	 */
    private DecElemento declaracao1;
    /**
     * Segunda declarao de classe, que pode ser simples ou composta.
     */
    private DecElemento declaracao2;
	/**
	 * Construtor.
	 * @param parametro1 Primeira declarao de classe.
	 * @param parametro2 Segunda parte da declarao de classe composta.
	 */
    public  DecElementoComposta(DecElemento declaracao1, DecElemento declaracao2){
        this.declaracao1 = declaracao1;
        this.declaracao2 = declaracao2;
    }
    /**
     * Cria um mapeamento do identificador para esta declarao 
     * de classe no ambiente  de execucao
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     *  e valores.
     * @return o ambiente modificado pela inicializao da classe.
     */
    public AmbienteExecucaoOO1 elabora(AmbienteExecucaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException,
               ElementoNaoDeclaradoException, ElementoJaDeclaradoException,
               ProcedimentoNaoDeclaradoException,ProcedimentoJaDeclaradoException{
        
    	return declaracao2.elabora(declaracao1.elabora(ambiente));
    }
    /**
     * Verifica se a declarao est bem tipada, ou seja, se as duas
     * declaraes esto bem tipadas
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     *  e seus tipos.
     * @return <code>true</code> se os tipos da declarao so vlidos;
     *          <code>false</code> caso contrario.
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
        throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
        ElementoJaDeclaradoException, ElementoNaoDeclaradoException,
        ProcedimentoNaoDeclaradoException,ProcedimentoJaDeclaradoException{

    	return declaracao1.checaTipo(ambiente) && declaracao2.checaTipo(ambiente);
    }
}
