package plp.orientadaObjetos1.declaracao.procedimento;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.expressao.leftExpression.Id;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.util.Tipo;
/**
 * Classe que representa uma declarao de parmetros.
 */
public class DecParametro {
    /**
     * Identificador declarado.
     */
    private Id id;
    /**
     * Tipo do identificador declarado.
     */
    private Tipo tipo;
    /**
     * Construtor.
     * @param id Identificador declarado.
     * @param tipo Tipo do identificador declarado.
     */
    public DecParametro(Id id, Tipo tipo){
        this.id = id;
        this.tipo = tipo;
    }
    /**
     * Obtm o identificador declarado.
     * @return o identificador.
     */
    public Id getId() {
        return id;
    }
    /**
     * Obtm o tipo do identifador declarado.
     * @return o tipo do identifador declarado.
     */
    public Tipo getTipo() {
        return tipo;
    }
    /**
     * Cria um mapeamento do identificador para o valor da expresso
     * desta declarao no AmbienteExecucao
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     *  e valores.
     * @return o ambiente modificado pela inicializao da varivel.
     */
    public AmbienteExecucaoOO1 elabora(AmbienteExecucaoOO1 ambiente) {
        return ambiente;
    }
    /**
     * Verifica se a declarao est bem tipada, ou seja, se a
     * expresso de inicializao est bem tipada.
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     *  e seus tipos.
     * @return <code>true</code> se os tipos da declarao so vlidos;
     *          <code>false</code> caso contrario.
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)throws ElementoNaoDeclaradoException {
        return tipo.eValido(ambiente);
    }

    /**
     * Cria um mapeamento do identificador para o tipo do parametro
     * desta declarao no AmbienteCompilacao
     *
     * @param ambiente o ambiente que contem o mapeamento entre identificador
     *  e seu tipo.
     * @return o ambiente modificado pela declarao do parametro.
     */
    public AmbienteCompilacaoOO1 declaraParametro(AmbienteCompilacaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException {

        ambiente.map(id, tipo);
        return ambiente;
    }
}
