package plp.orientadaObjetos1.declaracao.procedimento;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.comando.Comando;
import plp.orientadaObjetos1.comando.Procedimento;
import plp.orientadaObjetos1.excecao.declaracao.ElementoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ProcedimentoNaoDeclaradoException;
import plp.orientadaObjetos1.expressao.leftExpression.Id;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
/**
 * Representa uma declarao de procedimento simples.
 */
public class DecProcedimentoSimples implements  DecProcedimento {
    /**
     * Identificador do procedimento.
     */
    private Id nome;
    /**
     * Parmetros do procedimento.
     */
    private ListaDeclaracaoParametro parametrosFormais;
    /**
     * Comando, que pode ser simples ou composto, executado pelo procedimento.
     */
    private Comando comando;
    /**
     * Construtor.
     * @param nome Nome do procedimento.
     * @param parametrosFormais Parmetros do procedimento.
     * @param comando Comando(s) executado(s) pelo procedimento.
     */
    public DecProcedimentoSimples(Id nome, ListaDeclaracaoParametro parametrosFormais,Comando comando){
        this.nome = nome;
        this.parametrosFormais = parametrosFormais;
        this.comando = comando;
    }
    /**
     * Obtm o procedimento representado por nome.
     * @param nome O nome do procedimento procurado.
     * @return o procedimento identificado por nome.
     * @throws ProcedimentoNaoDeclaradoException quando no existe nenhum
     * procedimento declarado com esse nome.
     */
    public Procedimento getProcedimento(Id nome) throws ProcedimentoNaoDeclaradoException {
        if(this.nome.equals(nome)){
            return new Procedimento(parametrosFormais, comando);
        }
        else {
            throw new ProcedimentoNaoDeclaradoException(nome);
        }
    }
    /**
     * Verifica se a declarao est bem tipada, ou seja, se os
     * comandos esto bem tipados.
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     *  e seus tipos.
     * @return <code>true</code> se os tipos dos comandos so vlidos;
     *          <code>false</code> caso contrario.
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
       throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
              ProcedimentoJaDeclaradoException, ProcedimentoNaoDeclaradoException,
              ElementoNaoDeclaradoException,ElementoJaDeclaradoException {
       boolean resposta;
        if(parametrosFormais.checaTipo(ambiente)) {
            ambiente.mapParametrosProcedimento(nome, parametrosFormais);
            ambiente.incrementa();
            ambiente = parametrosFormais.declaraParametro(ambiente);
            resposta = comando.checaTipo(ambiente);
            ambiente.restaura();
        }
        else {
            resposta = false;
        }
        return resposta;
    }
}
