package plp.orientadaObjetos1.declaracao.procedimento;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.imperative1.util.Lista;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
/**
 * Um conjunto de declaraes de parmetro.
 */
public class ListaDeclaracaoParametro extends Lista<DecParametro>{
    /**
     * Construtor.
     */
    public ListaDeclaracaoParametro(){
       
    }
    /**
     * Construtor
     * @param declaracao A declarao contida por esta tail.
     */
    public ListaDeclaracaoParametro(DecParametro declaracao){
        super(declaracao, null);
    }
    /**
     * Construtor.
     * @param declaracao A declarao contida por esta tail.
     * @param listaDeclaracao A tail de declaraes que segue declarao.
     */
    public ListaDeclaracaoParametro(DecParametro declaracao, ListaDeclaracaoParametro listaDeclaracao){
        super(declaracao,listaDeclaracao);
    }
 
    /**
     * Cria um mapeamento do identificador para esta tail de declaraes
     * de parmetro.
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     * e valores.
     * @return o ambiente modificado pela declarao da classe.
     */
    public AmbienteExecucaoOO1 elabora(AmbienteExecucaoOO1 ambiente) {
        AmbienteExecucaoOO1 resposta;
        if(getHead() != null) {
            if(getTail() != null) {
                resposta = ((ListaDeclaracaoParametro)getTail()).elabora(getHead().elabora(ambiente));
            }
            else {
                resposta = getHead().elabora(ambiente);
            }
        }
        else {
            resposta = ambiente;
        }
        return resposta;
    }
    /**
     * Verifica se a declarao e a tail de declarao esto bem tipadas, ou seja, se a
     * expresso de inicializao est bem tipada.
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     *  e seus tipos.
     * @return <code>true</code> se os tipos da declarao so vlidos;
     *          <code>false</code> caso contrario.
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, ElementoNaoDeclaradoException {
        boolean resposta;
        if(getHead() != null) {
            if(getTail() != null) {
                resposta = getHead().checaTipo(ambiente) &&
				 			((ListaDeclaracaoParametro)getTail()).checaTipo(ambiente);
            }
            else {
                resposta = getHead().checaTipo(ambiente);
            }
        }
        else {
            resposta = true;
        }
        return resposta;
    }

    /**
     * Cria um mapeamento do identificador para o tipo do parametro
     * desta declarao no AmbienteCompilacao
     *
     * @param ambiente o ambiente que contem o mapeamento entre identificador
     *  e seu tipo.
     * @return o ambiente modificado pela declarao do parametro.
     */
    public AmbienteCompilacaoOO1 declaraParametro(AmbienteCompilacaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException {

        AmbienteCompilacaoOO1 resposta;
        if(getHead() != null) {
            if(getTail() != null) {
                resposta = ((ListaDeclaracaoParametro)getTail()).declaraParametro(getHead().declaraParametro(ambiente));
            }
            else {
                resposta = getHead().declaraParametro(ambiente);
            }
        }
        else {
            resposta = ambiente;
        }
        return resposta;
    }


}
