package plp.orientadaObjetos1.declaracao.variavel;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.expressao.leftExpression.Id;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.util.Tipo;

/**
 * Declarao de varivel composta.
 */
public class CompostaDecVariavel implements DecVariavel{
    /**
     * Primeira declarao de varivel.
     */
    private DecVariavel declaracao1;
    /**
     * Restante da declarao de variveis.
     */
    private DecVariavel declaracao2;

    /**
     * Construtor.
     * @param parametro1 Primeira declaraao de varivel.
     * @param parametro2 Restante da declarao de variveis.
     */
    public  CompostaDecVariavel(DecVariavel declaracao1, DecVariavel declaracao2){
        this.declaracao1 = declaracao1;
        this.declaracao2 = declaracao2;
    }
    /**
     * Obtm o tipo de uma varivel nessa declarao.
     * @param id O identificador da varivel.
     * @return o tipo deste identificador nessa declaraao.
     * @throws VariavelNaoDeclaradaException Quando nao h nenhuma varivel
     * declarada com esse id na declarao.
     */
    public Tipo getTipo(Id id) throws VariavelNaoDeclaradaException {
        Tipo tipo;
        try {
            tipo = declaracao1.getTipo(id);
        }
        catch(VariavelNaoDeclaradaException e) {
            tipo = declaracao2.getTipo(id);
        }
        return tipo;

    }

    /**
     * Cria um mapeamento dos identificadores para os valores das
     * expresses desta declarao composta no AmbienteExecucao
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     *  e valores.
     * @return o ambiente modificado pelas inicializaes das variveis.
     */
    public AmbienteExecucaoOO1 elabora(AmbienteExecucaoOO1 ambiente)
        throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
              ObjetoNaoDeclaradoException, ObjetoJaDeclaradoException,
              ElementoJaDeclaradoException, ElementoNaoDeclaradoException{
        return declaracao2.elabora(declaracao1.elabora(ambiente));
    }

    /**
     * Verifica se as declaraes esto bem tipadas, ou seja, se as
     * expresses de inicializao esto bem tipadas.
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     *  e seus tipos.
     * @return <code>true</code> se os tipos da declarao so vlidos;
     *          <code>false</code> caso contrario.
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException,
               ElementoNaoDeclaradoException, ElementoJaDeclaradoException {
        return declaracao1.checaTipo(ambiente) && declaracao2.checaTipo(ambiente);
    }
}
