package plp.orientadaObjetos1.declaracao.variavel;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.comando.New;
import plp.orientadaObjetos1.excecao.declaracao.ElementoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.expressao.leftExpression.Id;
import plp.orientadaObjetos1.expressao.valor.ValorNull;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.memoria.DefAgente;
import plp.orientadaObjetos1.memoria.DefElemento;
import plp.orientadaObjetos1.util.Tipo;
import plp.orientadaObjetos1.util.TipoElemento;

/**
 * Classe que representa a declaraao de uma varivel do tipo objeto.
 */
public class DecVariavelObjeto implements DecVariavel{
    /**
     * Tipo da varivel declarado.
     */
    private Tipo tipo;
    /**
     * Identificador representando o objeto.
     */
    private Id objeto;
    /**
     * Idenficador representando a classe da qual objeto  uma instncia.
     */
    private Id classe;
    /**
     * Construtor.
     * @param tipo Tipo declarado da varivel.
     * @param objeto Identificador do objeto.
     * @param classe Classe da qual objeto  uma instncia.
     */
    public DecVariavelObjeto(Tipo tipo, Id objeto, Id classe){
        this.tipo = tipo;
        this.objeto = objeto;
        this.classe = classe;
    }

    /**
     * Retorna o tipo do identificador a ser declarado no AmbienteCompilacao
     * @param id o identificador  da declaracao
     * @return o tipo do identificador
     */
    public Tipo getTipo(Id id) throws VariavelNaoDeclaradaException{
        if(this.objeto.equals(id)){
            return tipo;
        }
        else {
            throw new VariavelNaoDeclaradaException(id);
        }
    }

    /**
     * Cria um mapeamento do identificador para o objeto no ambiente de
     * execuo.
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     *  e valores.
     * @return o ambiente modificado pela inicializao da varivel.
     */
    public AmbienteExecucaoOO1 elabora(AmbienteExecucaoOO1 ambiente)
        throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
               ElementoJaDeclaradoException, ElementoNaoDeclaradoException,
               ObjetoJaDeclaradoException, ObjetoNaoDeclaradoException{
    	
    	AmbienteExecucaoOO1 aux = ambiente;
    	aux =  new SimplesDecVariavel(tipo, objeto, new ValorNull()).elabora(ambiente);
   		aux = new New(objeto, classe).executar(aux);       
        return aux;
    }

    /**
     *
     * Verifica se o tipo da classe associada  vlido (se existe).
     * @param ambiente o ambiente que contem o mapeamento entre objetos
     *  e suas classes.
     * @return <code>true</code> a classe existe
     *          <code>false</code> caso contrario.
     *
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
        throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
               ElementoJaDeclaradoException, ElementoNaoDeclaradoException{
        boolean resposta = false;
        
        DefElemento elemento = ambiente.getDefElemento(this.tipo.getTipo());
        TipoElemento tpClasse = new TipoElemento(classe, TipoElemento.CLASSE);
        
        if (elemento instanceof DefAgente) {
        	tpClasse.setTipo(TipoElemento.AGENTE);
        }
        
        
        if (tpClasse.eValido(ambiente) && tipo.eValido(ambiente) ) {
            resposta = tpClasse.equals(tipo);
            ambiente.map(objeto, tpClasse);
        }
        return resposta;
  }
}