package plp.orientadaObjetos1.declaracao.variavel;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.expressao.Expressao;
import plp.orientadaObjetos1.expressao.leftExpression.Id;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.util.Tipo;
import plp.orientadaObjetos1.util.TipoElemento;

/**
 * Classe que representa uma declaraao de varivel simples.
 */
public class SimplesDecVariavel implements DecVariavel{
    /**
     * Tipo da varivel declada.
     */
    private Tipo tipo;
    /**
     * Varivel declarada.
     */
    private Id id;
    /**
     * Expresso cujo valor ser atribudo  varivel.
     */
    private Expressao expressao;
    /**
     * Construtor.
     * @param tipo Tipo da varivel declarada.
     * @param id Varivel declarada.
     * @param expressao Expresso cujo valor ser atribudo  varivel.
     */
    public SimplesDecVariavel(Tipo tipo, Id id, Expressao expressao){
        this.tipo = tipo;
        this.id = id;
        this.expressao = expressao;
    }

    /**
    * Retorna o tipo do identificador a ser declarado no AmbienteCompilacao
    * @param id o identificador  da declaracao
    * @return o tipo do identificador
    */
    public Tipo getTipo(Id id) throws VariavelNaoDeclaradaException{
           if(this.id.equals(id)){
               return tipo;
           }
           else {
               throw new VariavelNaoDeclaradaException(id);
           }
    }

    /**
     * Cria um mapeamento do identificador para o valor da expresso
     * desta declarao no AmbienteExecucao
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     *  e valores.
     * @return o ambiente modificado pela inicializao da varivel.
     * @throws ElementoNaoDeclaradoException 
     *
     */
    public AmbienteExecucaoOO1 elabora(AmbienteExecucaoOO1 ambiente)
        throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
               ObjetoNaoDeclaradoException,ObjetoJaDeclaradoException, ElementoNaoDeclaradoException  {
        ambiente.map(id , expressao.avaliar(ambiente));
        return ambiente;
    }

    /**
     * Verifica se a declarao est bem tipada, ou seja, se a
     * expresso de inicializao est bem tipada.
     * @param ambiente o ambiente que contem o mapeamento entre identificadores
     *  e seus tipos.
     * @return <code>true</code> se a expresso  bem tipada;
     *          <code>false</code> caso contrario.
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException,
               ElementoNaoDeclaradoException {
        boolean resposta = false;
        if ( expressao.checaTipo(ambiente) ) {
            if( tipo instanceof TipoElemento) {
                resposta = expressao.getTipo(ambiente).equals(TipoElemento.TIPO_NULL) ||
                           expressao.getTipo(ambiente).equals(tipo);
            }
            else {
                resposta = expressao.getTipo(ambiente).equals(tipo);
            }
        }
        if(resposta) {
            ambiente.map(id, tipo);
        }
        return resposta;
    }
}
