package plp.orientadaObjetos1.expressao.binaria;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.expressao.Expressao;
import plp.orientadaObjetos1.expressao.valor.Valor;
import plp.orientadaObjetos1.expressao.valor.ValorString;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.util.Tipo;
import plp.orientadaObjetos1.util.TipoPrimitivo;


/**
* Um objeto desta classe representa uma expressao de Concatenacao entre
* objetos <code>ValorString</code>
*/
public class ExpConcat extends ExpBinaria{


    /**
     * Controi uma expressao de Concatenacao com as sub-expressoes especificadas.
     * Estas sub-expressoes devem ser tais que a avaliacao das mesmas resulta
     * em <code>ValorString</code>
     *
     * @param esq expressao da esquerda
     * @param dir expressao da direita
     */
    public ExpConcat(Expressao esq, Expressao dir){
        super(esq, dir, "++");
    }

    /**
     * Retorna o valor da expressao de Concatenacao
     */
    public Valor avaliar(AmbienteExecucaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException,
            ObjetoNaoDeclaradoException, ElementoNaoDeclaradoException {
        return obterResultadoDaConcatenacao(ambiente);
    }

    /**
     * Realiza a verificacao de tipos desta expressao.
     *
     * @param ambiente o ambiente de compilao.
     * @return <code>true</code> se os tipos da expressao so vlidos;
     *          <code>false</code> caso contrario.
     * @exception VariavelNaoDeclaradaException se existir um identificador
     *          nao declarado no ambiente.
     * @exception VariavelNaoDeclaradaException se existir um identificador
     *          declarado mais de uma vez no mesmo bloco do ambiente.
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, ElementoNaoDeclaradoException {
        boolean result;
        if (super.checaTipo(ambiente) &&
            ((TipoPrimitivo)getEsq().getTipo(ambiente)).eString() ||//we changed && to ||
            ((TipoPrimitivo)getDir().getTipo(ambiente)).eString()) {
            result = true;
        }
        else {
            result = false;
        }
        return result;
    }

    /**
     * Retorna os tipos possiveis desta expressao.
     *
     * @param ambiente o ambiente de compilao.
     * @return os tipos possiveis desta expressao.
     */
    public Tipo getTipo(AmbienteCompilacaoOO1 ambiente) {
        return TipoPrimitivo.TIPO_STRING;
    }

    /**
     * Retorna o valor inteiro que representa o resultado da concatenacao de dois Strings
     * @param ambiente  o Ambiente de Execuo
     * @return o valor inteiro que representa o resultado da concatenacao de dois Strings
     * @throws ElementoNaoDeclaradoException 
     */
    private ValorString obterResultadoDaConcatenacao(AmbienteExecucaoOO1 ambiente)
            throws VariavelJaDeclaradaException, VariavelNaoDeclaradaException,
                   ObjetoNaoDeclaradoException, ElementoNaoDeclaradoException{
        return new ValorString(
                    (((Valor)getEsq().avaliar(ambiente))).toString() +
                    (((Valor) getDir().avaliar(ambiente))).toString());
    }
}