package plp.orientadaObjetos1.expressao.leftExpression;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.expressao.Expressao;
import plp.orientadaObjetos1.expressao.valor.Valor;
import plp.orientadaObjetos1.expressao.valor.ValorRef;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.memoria.DefClasse;
import plp.orientadaObjetos1.memoria.DefElemento;
import plp.orientadaObjetos1.memoria.Objeto;
import plp.orientadaObjetos1.util.Tipo;
import plp.orientadaObjetos1.util.TipoElemento;

/**
 * Classe que representa um acesso de atributo a partir de uma expressao.
 */
public class AcessoAtributoId extends AcessoAtributo{
    /**
     * Expressao que acessa o atributo.
     */
    private LeftExpression av;
    /**
     * Construtor.
     * @param av Expressao do lado esquerdo, que acessa o atributo.
     * @param id O atributo sendo acessado.
     */
    public AcessoAtributoId(LeftExpression av, Id id){
        super(id);
        this.av = av;
    }
    /**
     * Avalia esse acesso de atributo obtendo o valor do atributo no ambiente.
     * @param ambiente o ambiente de execuo, que apresenta o mapeamento de
     * identificadores a valores.
     * @return o valor do atributo acessado no ambiente.
     * @throws VariavelNaoDeclaradaException
     * @throws VariavelJaDeclaradaException
     * @throws ObjetoNaoDeclaradoException
     */
    public Valor avaliar(AmbienteExecucaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException,
               ObjetoNaoDeclaradoException, ElementoNaoDeclaradoException{
        return obterValorDeIdNoAmbiente(ambiente);
    }
    /**
     * Obtm a expresso acessadora do atributo.
     * @return a expresso acessadora do atributo.
     */
    public Expressao getExpressaoObjeto(){
        return av;
    }
    /**
     * Verifica se os atributos associados foram declarados e se seus tipos
     * existem no ambiente.
     * @param ambiente o ambiente de compilao, com o mapeamento de identificadores
     * a tipos.
     * @return true, se as variveis acessadas j foram declaradas e seus
     * tipos existem.
     * @throws VariavelNaoDeclaradaException
     * @throws ElementoNaoDeclaradoException
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, ElementoNaoDeclaradoException{
        boolean resposta = false;
        if(av.checaTipo(ambiente)) {
            try{
                Tipo t = av.getTipo(ambiente);
                DefElemento defElemento = ambiente.getDefElemento(((TipoElemento)t).getTipo());
                if (defElemento instanceof DefClasse) {
                	((DefClasse)defElemento).getTipoAtributo(super.getId());
                	resposta = true;
                } else {
                	resposta = true;
                }                
            }
            catch(VariavelNaoDeclaradaException atrib){
                resposta = false;
            }
            catch(ElementoNaoDeclaradoException clas){
                resposta = false;
            }

        }
        return resposta;
    }
    /**
     * Obtm o tipo do atributo no ambiente.
     * @param ambiente que apresenta o mapeamento de identificadores a tipos.
     * @return o tipo do atributo acessado.
     * @throws VariavelNaoDeclaradaException
     * @throws ElementoNaoDeclaradoException
     */
    public Tipo getTipo(AmbienteCompilacaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, ElementoNaoDeclaradoException{
          //Logo abaixo obtenho a definicao da Classe (seus mtodos e atributos).
          //av.getTipo devera retornar uma instancia de TipoClasse e assim, TipoClasse.getTipo()
          //retorna o id (contendo o nome da classe) associado ao tipo dela
        Id nomeClasse = ((TipoElemento)av.getTipo(ambiente)).getTipo();
        DefClasse defClasse = (DefClasse) ambiente.getDefElemento(nomeClasse);
        Tipo tipoAtr = defClasse.getTipoAtributo(super.getId());
        //Em seguida retorno o tipo do atributo, caso ele esteja definido na classe.
        //caso no esteja, uma exceo ser lanada
        return tipoAtr;
    }
    /**
     * Obtm a expressao que acessa o atributo.
     * @return a LeftExpression que representa a expresso que acessa o atributo.
     */
    public LeftExpression getAv() {
        return av;
    }

    /**
     * Retorna o valor do Objeto representado por um certo id
     * @param ambiente  o Ambiente de Execuo
     * @return o valor do Objeto representado por um certo id
     * @throws ElementoNaoDeclaradoException 
     */
    private Valor obterValorDeIdNoAmbiente(AmbienteExecucaoOO1 ambiente)
            throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException,
               ObjetoNaoDeclaradoException, ElementoNaoDeclaradoException {
        // Pegando o objeto no ambiente
        ValorRef referencia = (ValorRef) av.avaliar(ambiente);
        Objeto objeto = ambiente.getObjeto(referencia);
        // Recuperando o mapeamento de valores do objeto (atributos do objeto)
        AmbienteExecucaoOO1 aux = objeto.getEstado();
        // Recuperando o valor do atributo "id" do objeto
        return aux.get(super.getId());
    }
}