package plp.orientadaObjetos1.expressao.leftExpression;

import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.expressao.Expressao;
import plp.orientadaObjetos1.expressao.This;
import plp.orientadaObjetos1.expressao.valor.Valor;
import plp.orientadaObjetos1.expressao.valor.ValorRef;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.memoria.DefClasse;
import plp.orientadaObjetos1.memoria.Objeto;
import plp.orientadaObjetos1.util.Tipo;

/**
 * Representa um acesso de atributo a partir de um objeto this.
 */
public class AcessoAtributoThis extends AcessoAtributo{
    /**
     * O objeto this.
     */
    private This varThis;
    /**
     * Construtor.
     * @param varThis O objeto this.
     * @param id O identificador sendo acessado.
     */
    public AcessoAtributoThis(This varThis, Id id){
        super(id);
        this.varThis = varThis;
    }
    /**
     * O valor do atributo acessado no ambiente.
     * @param ambiente o ambiente contendoo mapeamento de identificadores
     * a valores.
     * @return o valor do atributo acessado.
     * @throws VariavelNaoDeclaradaException
     * @throws VariavelJaDeclaradaException
     * @throws ObjetoNaoDeclaradoException
     */
    public Valor avaliar(AmbienteExecucaoOO1 ambiente)
        throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException,
        ObjetoNaoDeclaradoException, ElementoNaoDeclaradoException{
        return obterValorDeIdNoAmbiente(ambiente);
    }
    /**
     * Obtm a expresso que acessa o identificador.
     * @return a expressao que acessa o identificador.
     */
    public Expressao getExpressaoObjeto(){
        return varThis;
    }
    /**
     * Verifica se o this est associado a um objeto e se o atributo existe.
     * @param ambiente o ambiente com o mapeamento de identificadores a tipos.
     * @return true, se o this est associado a um objeto e se o atributo existe,
     * ou false, caso contrrio.
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 ambiente) {
        boolean resposta = false;
        try {
              resposta = varThis.checaTipo(ambiente);
              if(resposta) {
                  Tipo tipo = varThis.getTipo(ambiente);
                  DefClasse defClasse = (DefClasse) ambiente.getDefElemento(tipo.getTipo());
                  defClasse.getTipoAtributo(super.getId());
                  resposta = true;
              }
        }
        catch(VariavelNaoDeclaradaException atrib){
              resposta = false;
        }
        catch(ElementoNaoDeclaradoException clas){
              resposta = false;
        }
        return resposta;
    }
    /**
     * Obtm o tipo do atributo acessado.
     * @param ambiente o ambiente com o mapeamento de identificadores a tipos.
     * @return true, se foi associado um tipo a esse identificador acessado no
     * escopo corrente.
     * @throws VariavelNaoDeclaradaException
     * @throws ElementoNaoDeclaradoException
     */
    public Tipo getTipo(AmbienteCompilacaoOO1 ambiente)
        throws VariavelNaoDeclaradaException,ElementoNaoDeclaradoException{
        //Logo abaixo obtenho a definicao da Classe (seus mtodos e atributos).
        //this.getTipo() devera retornar uma instancia de TipoClasse e assim, TipoClasse.getTipo()
        //retorna o id (contendo o nome da classe) associado ao tipo dela
        DefClasse defClasse = (DefClasse) ambiente.getDefElemento(varThis.getTipo(ambiente).getTipo());
        //Em seguida retorno o tipo do atributo, caso ele esteja definido na classe.
        //caso no esteja, uma exceo ser lanada
        return defClasse.getTipoAtributo(super.getId());
    }

    /**
     * Retorna o valor do Objeto representado por um certo id
     * @param ambiente  o Ambiente de Execuo
     * @return o valor do Objeto representado por um certo id
     * @throws ElementoNaoDeclaradoException 
     */
    private Valor obterValorDeIdNoAmbiente(AmbienteExecucaoOO1 ambiente)
            throws VariavelNaoDeclaradaException, VariavelJaDeclaradaException,
               ObjetoNaoDeclaradoException, ElementoNaoDeclaradoException {
        // Pegando o objeto no ambiente
        ValorRef referencia = (ValorRef)varThis.avaliar(ambiente);
        Objeto objeto = ambiente.getObjeto(referencia);
        // Recuperando o mapeamento de valores do objeto (atributos do objeto)
        AmbienteExecucaoOO1 aux = objeto.getEstado();
        // Recuperando o valor do atributo "id" do objeto
        return aux.get(super.getId());
        //return ambiente.getObjeto((ValorRef)varThis.avaliar(ambiente)).getEstado().getValor(super.getId());
    }
}