package plp.orientadaObjetos1.expressao.valor;

import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.memoria.AmbienteExecucaoOO1;
import plp.orientadaObjetos1.util.Tipo;
import plp.orientadaObjetos1.util.TipoPrimitivo;
/**
 * Classe que representa a referncia a um objeto.
 */
public class ValorRef implements Valor{
    /**
     * Primeira referencia valida na memoria
     */
    public static final int VALOR_INICIAL = 0;
    /**
     * Valor que representa a referncia.
     */
    private int valor;
    /**
     * Construtor.
     * @param valor O valor que representa a referncia.
     */
    public ValorRef(int valor){
        if ( valor >= ValorRef.VALOR_INICIAL){
             this.valor = valor;
    }
        else this.valor = VALOR_INICIAL;
    }

    /**
     * Obtm o valor.
     * @return o valor referncia.
     */
    public int valor() {
        return valor;
    }
    /**
     * Avalia um certo valor referncia.
     * @param ambiente o ambiente de execuao
     * @return o valor associado a uma dada referncia.
     */
    public Valor avaliar(AmbienteExecucaoOO1 ambiente) throws ElementoNaoDeclaradoException {
        return this;
    }

    //Os mtodos getTipo e checaTipo de ValorRef
    //no foram utilizados nessa linguagem.
    public int hashCode() {
        return valor;
    }
    /**
     * Obtm o tipo associado a este valor referncia no ambiente de compilaao.
     * @param amb o ambiente de compilao.
     * @return o tipo deste valor referncia.
     */
    public Tipo getTipo(AmbienteCompilacaoOO1 amb) {
        return TipoPrimitivo.TIPO_INTEIRO;
    }
    /**
     * Checa o tipo deste valor referencia no ambiente de compilao.
     * @param amb o ambiente de compilao
     * @return true em todos os casos.
     */
    public boolean checaTipo(AmbienteCompilacaoOO1 amb) {
        return true;
    }
    /**
     * Compara dois valores
     * @param val o valor que vai ser comparado com este.
     * @return true se os valores forem iguais e false, caso contrrio.
     */
    public boolean equals(Valor val){
        if ( val instanceof ValorRef)
            return valor == ((ValorRef)val).valor();
        else return false;
    }

    /**
     * Incrementa este valor referncia.
     * @return um novo valor referncia.
     */
    public ValorRef incrementa(){
        valor = valor + 1;
        return this;
    }
}
