package plp.orientadaObjetos1.memoria;

import java.util.HashMap;
import java.util.Stack;

import plp.expressions2.expression.Id;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.execucao.EntradaInvalidaException;
import plp.orientadaObjetos1.expressao.valor.Valor;
import plp.orientadaObjetos1.expressao.valor.ValorRef;
import plp.orientadaObjetos1.memoria.colecao.ListaValor;
import plp.orientadaObjetos1.util.Tipo;

/**
 * Classe que representa um ambiente de execuo, contendo om mapeamento
 * entre identificadores e valores.
 */
public interface AmbienteExecucaoOO1 extends AmbienteOO1<Valor> {
    /**
     * Obtm a pilha de valores associados a identificadores
     * @return a pilha de valores associados a identificadores.
     */
	public Stack<HashMap<Id, Valor>> getPilha();

    /**
     * Retorna a pilha com as definioes das classes.
     * @return a pilha com as definioes das classes.
     */
    public HashMap<Id, DefElemento> getMapDefElemento();

    /**
     * Obtm a pilha com os objetos e seus valores.
     * @return a pilha com os objetos e seus valores.
     */
    public Stack<HashMap<ValorRef, Objeto>> getPilhaObjeto();

    /**
     * Mapeia um valor referncia a um objeto.
     * @param valorRef Valor referncia.
     * @param objeto Objeto.
     * @throws ObjetoJaDeclaradoException Quando esse objeto j foi declarado.
     */
    public void mapObjeto(ValorRef valorRef, Objeto objeto) throws ObjetoJaDeclaradoException;

    /**
     * Altera o valor associado a um identificador.
     * @param idArg Identificador.
     * @param valorId O valor a ser associado ao identificador.
     * @throws VariavelNaoDeclaradaException Quando a varivel no foi
     * declarada.
     */
    public void changeValor(Id idArg, Valor valorId) throws VariavelNaoDeclaradaException;

    /**
     * Obtm o objeto associado a um dado valor referencia.
     * @param valorRef Valor referncia
     * @return o objeto associado a um dado valor referencia.
     * @throws ObjetoNaoDeclaradoException Quando o objeto no foi declarado.
     */
    public Objeto getObjeto(ValorRef valorRef) throws ObjetoNaoDeclaradoException;

    /**
     * Obtm a prxima referncia de acordo com o contexto atual de execuo.
     * @return a prxima referncia de acordo com o contexto atual de execuo.
     */
    public ValorRef getProxRef();

    /**
     * Obtm o valor referencia atual.
     * @return o  valor referencia atual.
     */
    public ValorRef getRef();

    /**
     * L da entrada padro e associa o contedo a um determinado identificador.
     * @param tipoIdLido Tipo do identificador ao qual ser associado o valor
     * lido.
     * @return o valor lido.
     * @throws EntradaInvalidaException Quando a entrada fornecida no pode
     * ser atribuda ao tipo do identificador.
     */
    public Valor read(Tipo tipoIdLido) throws EntradaInvalidaException;

    /**
     * Escreve um valor 'v' na sada.
     * @param v O valor a ser escrito.
     * @return o ambiente de execuo, que representa o estado atual.
     */
    public AmbienteExecucaoOO1 write(Valor v);

    /**
     * Obtm a entrada.
     * @return a entrada.
     */
    public ListaValor getEntrada();

    /**
     * Obtm a sada.
     * @return a sada.
     */
    public ListaValor getSaida();

    /**
     * Obtm um novo contexto de execuo com a mesma entrada, sada e pilha
     * de mapeamentos id/valor.
     * @return um novo contexto de execuo com a mesma entrada, sada e pilha
     * de mapeamentos id/valor.
     */
    public ContextoExecucaoOO1 getContextoIdValor();
}
