package plp.orientadaObjetos1.memoria;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Stack;

import plp.expressions2.expression.Id;
import plp.expressions2.memory.VariavelJaDeclaradaException;
import plp.expressions2.memory.VariavelNaoDeclaradaException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoJaDeclaradoException;
import plp.orientadaObjetos1.excecao.declaracao.ObjetoNaoDeclaradoException;
import plp.orientadaObjetos1.excecao.execucao.EntradaInvalidaException;
import plp.orientadaObjetos1.expressao.valor.Valor;
import plp.orientadaObjetos1.expressao.valor.ValorBooleano;
import plp.orientadaObjetos1.expressao.valor.ValorInteiro;
import plp.orientadaObjetos1.expressao.valor.ValorNull;
import plp.orientadaObjetos1.expressao.valor.ValorRef;
import plp.orientadaObjetos1.expressao.valor.ValorString;
import plp.orientadaObjetos1.memoria.colecao.ListaValor;
import plp.orientadaObjetos1.util.Tipo;
import plp.orientadaObjetos1.util.TipoPrimitivo;

public class ContextoExecucaoOO1 implements AmbienteExecucaoOO1 {

    /**
	 * A pilha de blocos de contexto.
	 */
    private Stack<HashMap<Id, Valor>> pilha;

    /**
	 * A pilha de classes do contexto.
	 */
    private HashMap<Id, DefElemento> mapDefElemento;

    /**
	 * A pilha de objetos de contexto.
	 */

    private Stack<HashMap<ValorRef, Objeto>> pilhaObjeto;

    /**
	 * A pilha de blocos de contexto.
	 */
    private ListaValor entrada;

    /**
	 * A pilha de blocos de contexto.
	 */
    private ListaValor saida;

    /**
	 * A referncia do objeto a ser inserido na pilha de objetos
	 */
    private ValorRef proxRef;

    /**
	 * Construtor utilizado quando queremos ler do teclado.
	 */
    public ContextoExecucaoOO1(){
        pilha = new Stack<HashMap<Id, Valor>>();

        pilhaObjeto = new Stack<HashMap<ValorRef, Objeto>>();       // esta
																	// pilha nao
																	// cresce
        pilhaObjeto.push(new HashMap<ValorRef, Objeto>());        // tem de
																	// incrementar
																	// no comeo

        mapDefElemento = new HashMap<Id, DefElemento>();    // criacao do mapeamento de classes
        
        this.entrada = null;
        this.saida = new ListaValor();
    }

    /**
	 * Construtor da classe.
	 */
    public ContextoExecucaoOO1(AmbienteExecucaoOO1 ambiente) throws VariavelJaDeclaradaException{
       proxRef = ambiente.getRef();
       this.pilhaObjeto = ambiente.getPilhaObjeto();
       this.mapDefElemento = ambiente.getMapDefElemento();
       this.entrada = ambiente.getEntrada();
       this.saida = ambiente.getSaida();
       pilha = new Stack<HashMap<Id, Valor>>();
	   HashMap<Id, Valor> aux = new HashMap<Id, Valor>();
       aux.put(new Id("this"), new ValorNull());
       pilha.push(aux);

  }

    /**
	 * Construtor.
	 * 
	 * @param entrada
	 *            Entrada para o contexto de execuo.
	 */
    public ContextoExecucaoOO1(ListaValor entrada){
        pilha = new Stack<HashMap<Id, Valor>>();

        pilhaObjeto = new Stack<HashMap<ValorRef, Objeto>>();       // esta
																	// pilha nao
																	// cresce
        pilhaObjeto.push(new HashMap<ValorRef, Objeto>());        // tem de
																	// incrementar
																	// no comeo

        mapDefElemento = new HashMap<Id, DefElemento>();    // inicializacao do map
        
        this.entrada = entrada;
        this.saida = new ListaValor();
    }

    /**
	 * Obtm a pilha de valores associados a identificadores
	 * 
	 * @return a pilha de valores associados a identificadores.
	 */
    public Stack<HashMap<Id, Valor>> getPilha(){
       return this.pilha;
    }

    /**
	 * Retorna a pilha com as definioes das classes.
	 * 
	 * @return a pilha com as definioes das classes.
	 */
    public HashMap<Id, DefElemento> getMapDefElemento() {
       return this.mapDefElemento;
    }

    /**
	 * Obtm a pilha com os objetos e seus valores.
	 * 
	 * @return a pilha com os objetos e seus valores.
	 */
    public Stack<HashMap<ValorRef, Objeto>> getPilhaObjeto(){
       return this.pilhaObjeto;
    }

    /**
	 * L da entrada padro e associa o contedo a um determinado identificador.
	 * 
	 * @param tipoIdLido
	 *            Tipo do identificador ao qual ser associado o valor lido.
	 * @return o valor lido.
	 * @throws EntradaInvalidaException
	 *             Quando a entrada fornecida no pode ser atribuda ao tipo do
	 *             identificador.
	 */
    public Valor read(Tipo tipoIdLido) throws EntradaInvalidaException {
      String valorLido = leEntrada();
      if (valorLido!=null){
        valorLido = valorLido.trim();
        if (tipoIdLido instanceof TipoPrimitivo){
            TipoPrimitivo tipo = (TipoPrimitivo) tipoIdLido;
            try {
              if (tipo.eBooleano()){
                  return new ValorBooleano(Boolean.parseBoolean(valorLido));
              } else if (tipo.eInteiro()){
                  return new ValorInteiro(Integer.parseInt(valorLido));
              } else if (tipo.eString()){
                  return new ValorString(valorLido);
              }
            }
            catch(NumberFormatException e){
              throw new EntradaInvalidaException("O tipo da entrada e o da varivel"+
                                                 " a ser lida so diferentes!");
            }
        }
      }
      throw new EntradaInvalidaException("O tipo da varivel a ser lida no  um tipo Primitivo!");
    }

    /**
	 * Este mtodo l uma entrada que pode ser de uma tail ou do teclado
	 * 
	 * @return Obtm uma entrada que pode ser de uma tail ou do teclado
	 * @exception Lana
	 *                uma exceo se a tail com os valores nao tiver mais
	 *                elementos.
	 */
    private String leEntrada() throws EntradaInvalidaException{
      if(this.entrada==null) {
        return leDaEntradaPadrao();
      } else {
        // Se nao tivermos mais nada na tail de valores
        if(entrada.length()==0) {
          throw new EntradaInvalidaException("Nmero de argumentos menor do que o nmero de reads!");
        }
        return leDaListaValor();
      }
    }
    /**
	 * Este mtodo l da entrada padro
	 * 
	 * @return o que o usurio digitou na entrada padro
	 */
    private String leDaEntradaPadrao(){
      try {
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
        return in.readLine();
      }
      catch (IOException e){
         System.out.println("Erro no valor lido da entrada padro");
      }
      return "";
    }

    /**
	 * Este mtodo l da entrada padro
	 * 
	 * @return o que o usurio digitou na entrada padro
	 */
    private String leDaListaValor(){
      String retorno = entrada.getHead().toString();
      entrada = (ListaValor)entrada.getTail();
      return retorno;
    }

    /**
	 * Obtm a sada.
	 * 
	 * @return a sada.
	 */
    public ListaValor getSaida() {
        return saida;
    }

    /**
	 * Obtm a entrada.
	 * 
	 * @return a entrada.
	 */
    public ListaValor getEntrada(){
        return entrada;
    }

    /**
	 * Escreve um valor 'v' na sada.
	 * 
	 * @param v
	 *            O valor a ser escrito.
	 * @return o ambiente de execuo, que representa o estado atual.
	 */
    public AmbienteExecucaoOO1 write(Valor v){
        saida.write(v);
        return this;
    }
    /**
	 * Incrementa a pilha do ambiente, passando para o prximo estado.
	 */
    public void incrementa() {
        pilha.push(new HashMap<Id, Valor>());
        // pilhaDefClasse.push(new HashIdDefClasse()); // s incrementa no
		// construtor
        // pilhaObjeto.push(new HashValorObjeto()); // s incrementa no
		// construtor
    }

    /**
	 * Restaura o estado do ambiente.
	 */
    public void restaura(){
        pilha.pop();
        // pilhaDefClasse.pop(); // no restaura
        // pilhaObjeto.pop(); // no restaura
    }

    /**
	 * Mapeia um identificador a um valor.
	 * 
	 * @param idArg
	 *            Identificador.
	 * @param valorId
	 *            Valor que vai ser associado ao identificador.
	 * @throws VariavelJaDeclaradaException
	 *             Quando a varivel j foi declarada.
	 */
    public void map(Id idArg, Valor valorId)
        throws VariavelJaDeclaradaException {
		HashMap<Id, Valor> aux = pilha.peek();
        if (aux.put(idArg, valorId) != null) {
            throw new VariavelJaDeclaradaException(idArg);
        }

    }

    /**
	 * Mapeia um identificador a um definio de classe.
	 * 
	 * @param idArg
	 *            o nome da classe
	 * @param defClasse
	 *            Definio da Classe.
	 * @throws ElementoJaDeclaradoException
	 *             quando a classe j foi declarada.
	 */
    public void mapDefElemento(Id idArg, DefElemento defElemento)
        throws ElementoJaDeclaradoException {
		if (this.mapDefElemento.put(idArg, defElemento) != null) {
            throw new ElementoJaDeclaradoException(idArg);
        }
    }

    /**
	 * Mapeia um valor referncia a um objeto.
	 * 
	 * @param valorRef
	 *            Valor referncia.
	 * @param objeto
	 *            Objeto.
	 * @throws ObjetoJaDeclaradoException
	 *             Quando esse objeto j foi declarado.
	 */
     public void mapObjeto(ValorRef valorRef, Objeto objeto)
        throws ObjetoJaDeclaradoException {
		 HashMap<ValorRef, Objeto> aux = pilhaObjeto.peek();
        if (aux.put(valorRef, objeto) != null) {
            throw new ObjetoJaDeclaradoException(objeto.getClasse());
        }
    }

    /**
	 * Altera o valor associado a um identificador.
	 * 
	 * @param idArg
	 *            Identificador.
	 * @param valorId
	 *            O valor a ser associado ao identificador.
	 * @throws VariavelNaoDeclaradaException
	 *             Quando a varivel no foi declarada.
	 */
    public void changeValor(Id idArg, Valor valorId)
        throws VariavelNaoDeclaradaException {
        Valor result = null;
        Stack<HashMap<Id, Valor>> auxStack = new Stack<HashMap<Id, Valor>>();
        while (result == null && !pilha.empty()) {
			HashMap<Id, Valor> aux = pilha.pop();
            auxStack.push(aux);
            result = aux.get(idArg);
            if(result != null) {
                aux.put(idArg, valorId);
                break;
            }
        }
        while (!auxStack.empty()) {
            pilha.push(auxStack.pop());
        }
        if (result == null) {
            throw new VariavelNaoDeclaradaException(idArg);
        }
    }

    /**
	 * Obtm o valor associado a um determinado identificador.
	 * 
	 * @param idArg
	 *            Identificador
	 * @return o valor associado a um determinado identificador.
	 * @throws VariavelNaoDeclaradaException
	 *             Quando a varivel no foi declarada.
	 */
    public Valor get( Id idArg )
        throws VariavelNaoDeclaradaException {
        Valor result = null;
        Stack<HashMap<Id, Valor>> auxStack = new Stack<HashMap<Id, Valor>>();
        while (result == null && !pilha.empty()) {
            HashMap<Id, Valor> aux = pilha.pop();
            auxStack.push(aux);
            result = aux.get(idArg);
        }
        while (!auxStack.empty()) {
            pilha.push(auxStack.pop());
        }
        if (result == null) {
            throw new VariavelNaoDeclaradaException(idArg);
        } else {
            return result;
        }
    }


    /**
	 * Obtm a definio da classe cujo nome  idArg
	 * 
	 * @param idArg
	 *            Nome da classe.
	 * @return a definio da classe.
	 * @throws ElementoNaoDeclaradoException
	 *             quando nao foi declarada nenhuma classe com esse nome.
	 */
    public DefElemento getDefElemento(Id idArg)
        throws ElementoNaoDeclaradoException  {
        DefElemento result = null;
        result = this.mapDefElemento.get(idArg);
        return result;
    }

    /**
	 * Obtm o objeto associado a um dado valor referencia.
	 * 
	 * @param valorRef
	 *            Valor referncia
	 * @return o objeto associado a um dado valor referencia.
	 * @throws ObjetoNaoDeclaradoException
	 *             Quando o objeto no foi declarado.
	 */
    public Objeto getObjeto(ValorRef valorRef)
        throws ObjetoNaoDeclaradoException {
        Objeto result = null;
        Stack<HashMap<ValorRef, Objeto>> auxStack = new Stack<HashMap<ValorRef, Objeto>>();
        while (result == null && !pilhaObjeto.empty()) {
			HashMap<ValorRef, Objeto> aux = pilhaObjeto.pop();
            auxStack.push(aux);
            result = aux.get(valorRef);
        }
        while (!auxStack.empty()) {
            pilhaObjeto.push(auxStack.pop());
        }
        if (result == null) {
            throw new ObjetoNaoDeclaradoException(new Id(valorRef.toString()));
        } else {
            return result;
        }
    }

    /**
	 * Obtm a prxima referncia de acordo com o contexto atual de execuo.
	 * 
	 * @return a prxima referncia de acordo com o contexto atual de execuo.
	 */
    public ValorRef getProxRef() {
        ValorRef aux = new ValorRef(proxRef.valor());
        proxRef = proxRef.incrementa();
        return aux;
    }

    /**
	 * Obtm o valor referencia atual.
	 * 
	 * @return o valor referencia atual.
	 */
    public ValorRef getRef() {
        if (proxRef == null)
            proxRef = new ValorRef(ValorRef.VALOR_INICIAL);
          return proxRef;
    }

    /**
	 * Retorna a representao textual do contexto de execuo.
	 * 
	 * @return a representao textual do contexto de execuo.
	 */
    public String toString() {
        String resposta = null;
        Valor valor = null;
        Objeto objeto = null;
        Stack<HashMap<Id, Valor>> auxStack = new Stack<HashMap<Id, Valor>>();
        Stack<HashMap<ValorRef, Objeto>> auxStackObjeto = new Stack<HashMap<ValorRef, Objeto>>();

        while (!pilha.empty()) {
			HashMap<Id, Valor> aux = pilha.pop();
            auxStack.push(aux);
			for (Id id : aux.keySet()) {
                valor = aux.get(id);
                resposta = id + " " + valor + "\n";
            }
        }
        while (!auxStack.empty()) {
            pilha.push(auxStack.pop());
        }
        while (!pilhaObjeto.empty()) {
			HashMap<ValorRef, Objeto> aux = pilhaObjeto.pop();
            auxStackObjeto.push(aux);
            for(ValorRef valorRef : aux.keySet()) {
                objeto = aux.get(valorRef);
                resposta = valorRef + " " + objeto + "\n";
            }
        }
        while (!auxStackObjeto.empty()) {
            pilhaObjeto.push(auxStackObjeto.pop());
        }
        return resposta;
    }

    /**
	 * Obtm um novo contexto de execuo com a mesma entrada, sada e pilha de
	 * mapeamentos id/valor.
	 * 
	 * @return um novo contexto de execuo com a mesma entrada, sada e pilha
	 *         de mapeamentos id/valor.
	 */
    public ContextoExecucaoOO1 getContextoIdValor() {
        ContextoExecucaoOO1 ambiente = new ContextoExecucaoOO1(this.getEntrada());
        ambiente.pilha = this.pilha;
        ambiente.saida = this.saida;
        return ambiente;
    }

}