package plp.orientadaObjetos1.util;

import plp.orientadaObjetos1.memoria.AmbienteCompilacaoOO1;
import plp.orientadaObjetos1.excecao.declaracao.ElementoNaoDeclaradoException;
import plp.orientadaObjetos1.expressao.leftExpression.Id;

/**
 * Classe que representa os possiveis tipos de uma expressao.
 */
public class TipoElemento implements Tipo {

    /**
     * Indica que a expressao associada &eacute; uma classe.
     */
    private Id tipoElemento;
    
    private int tipo;

    /**
     * Indica que a expressao associada &eacute; nula.
     */
    public static final Id NULL = new Id("NULL");

    /**
     * Constante de tipo nulo.
     */
    public static final Tipo TIPO_NULL = new TipoElemento(NULL, -1);
    
    public static final int CLASSE = 1;
    
    public static final int AGENTE = 2;
   

     /**
     * Construtor da classe.
     *
     * @param tipo o tipo da expressao associada.
     */

     public TipoElemento(Id tipoElemento, int tipo) {
        this.tipoElemento = tipoElemento;
        this.tipo = tipo;
     }


    /**
     * Retorna o tipo da expressao associada.
     *
     * @return o tipo da expressao associada.
     */
    public Id getTipo() {
        return tipoElemento;
    }


    /**
     * Indica se esta classe &eacute; um tipo vlido, ou seja, se j tiver sido declarada.
     *
     * @return <code>true</code> se esta classe for um tipo vlido (j declarada);
     *          <code>false</code> caso contrario.
     */

    public boolean eValido(AmbienteCompilacaoOO1 ambiente)
         throws ElementoNaoDeclaradoException {
        boolean resposta = false;
        try{
             resposta =  (tipoElemento == NULL) || (ambiente.getDefElemento(tipoElemento) != null);
         }
         catch(ElementoNaoDeclaradoException c){
              resposta = false;
         }
         return resposta;
    }
    
    public boolean isClasse() {
    	return this.tipo == CLASSE;
    }
    
    public boolean isAgente() {
    	return this.tipo == AGENTE;
    }

    /**
     * Compara este tipo com o tipo dado.
     *
     * @return <code>true</code> se se tratarem do mesmo tipo;
     *          <code>false</code> caso contrario.
     */
    public boolean equals(Object obj) {
        return (obj instanceof TipoElemento) &&
               ((TipoElemento)obj).tipoElemento.equals(this.tipoElemento);
    }
    /**
     * Retorna  a descrio textual do tipo.
     * @return  a descrio textual do tipo.
     */
    public String toString() {
        return tipoElemento.toString();
    }


	public void setTipo(int tipo) {
		this.tipo = tipo;
	}
}